@@ -109,6 +109,12 @@ void HttpLog::init()
 }
 
 
+void HttpLog::updateLogPatternWithPid(int pid)
+{
+    lsnprintf( s_logPattern, sizeof(s_logPattern), "%%d [%%p] [%d] %%m", pid);
+}
+
+
 bool HttpLog::isEnabled(Logger *pLogger, int level)
 {
     if (pLogger)
@@ -150,6 +156,12 @@ void HttpLog::toggleDebugLog()
 }
 
 
+int  HttpLog::getLogLevel()
+{
+    return logger()->getLevel();
+}
+
+
 void HttpLog::setLogLevel(int level)
 {
     logger()->setLevel(level);
