@@ -1,6 +1,6 @@
 /*****************************************************************************
 *    Open LiteSpeed is an open source HTTP server.                           *
-*    Copyright (C) 2013 - 2018  LiteSpeed Technologies, Inc.                 *
+*    Copyright (C) 2013 - 2020  LiteSpeed Technologies, Inc.                 *
 *                                                                            *
 *    This program is free software: you can redistribute it and/or modify    *
 *    it under the terms of the GNU General Public License as published by    *
@@ -63,16 +63,30 @@ SubstItem::~SubstItem()
 {
     if (m_value.m_pStr)
     {
-        if ((m_type == REF_STRING) || (m_type == REF_ENV))
+        switch (m_type)
+        {
+        case REF_STRING:
+        case REF_ENV:
+        case REF_HTTP_HEADER:
+        case REF_SSI_VAR:
+        case REF_RESP_HEADER:
             delete m_value.m_pStr;
-        if (m_type == REF_FORMAT_STR)
+            break;
+        case REF_FORMAT_STR:
             delete(SubstFormat *)m_value.m_pAny;
+            break;
+        case REF_EXPR:
+            delete(Expression *)m_value.m_pAny;
+            break;
+        }
     }
 }
 
 
 SubstItem::SubstItem(const SubstItem &rhs)
-    : LinkedObj(), m_type(rhs.m_type)
+    : LinkedObj()
+    , m_type(rhs.m_type)
+    , m_subType(0)
 {
     if (rhs.m_value.m_pStr)
     {
@@ -80,6 +94,8 @@ SubstItem::SubstItem(const SubstItem &rhs)
         {
         case REF_STRING:
         case REF_ENV:
+        case REF_SSI_VAR:
+        case REF_RESP_HEADER:
             m_value.m_pStr = new AutoStr2(*rhs.m_value.m_pStr);
             break;
         case REF_FORMAT_STR:
@@ -549,7 +565,7 @@ int RequestVars::getReqVar(HttpSession *pSession, int type, char *&pValue,
         return pReq->getQueryStringLen();
     case REF_AUTH_TYPE:
         //TODO: hard code for now
-        strncpy(pValue, "Basic", 6);
+        lstrncpy(pValue, "Basic", bufLen);
         return 5;
     case REF_REQUEST_FN:
     case REF_SCRIPTFILENAME:
@@ -603,7 +619,7 @@ int RequestVars::getReqVar(HttpSession *pSession, int type, char *&pValue,
                     do {
                         retry = 0;
                         buffer = (char *)malloc(bufsize);
-                        if (buffer != NULL) {                    
+                        if (buffer != NULL) {
                             errno = 0;
                             if ((getpwuid_r(st.st_uid, &pw, buffer, bufsize, &ppw) == -1) &&
                                 (errno == ERANGE)) {
@@ -623,7 +639,7 @@ int RequestVars::getReqVar(HttpSession *pSession, int type, char *&pValue,
                 else
                 {
                     struct group gr;
-                    char *buffer;        
+                    char *buffer;
                     struct group *pgr;
                     ssize_t bufsize;
                     int retry = 0;
@@ -635,7 +651,7 @@ int RequestVars::getReqVar(HttpSession *pSession, int type, char *&pValue,
                         buffer = (char *)malloc(bufsize);
                         if (buffer != NULL) {
                             errno = 0;
-                            if ((getgrgid_r(st.st_gid, &gr, buffer, sizeof(buffer), &pgr) == -1) &&
+                            if ((getgrgid_r(st.st_gid, &gr, buffer, bufsize, &pgr) == -1) &&
                                 (errno == ERANGE)) {
                                 bufsize *= 2;
                                 retry = 1;
@@ -1066,6 +1082,8 @@ const char *RequestVars::getCookieValue(HttpReq *pReq,
 #ifdef TEST_NEW_FUN
     //TODO: do some test right now, use these code to return the specified cookie
     cookieval_t *cookie = pReq->getCookie(pCookieName, nameLen);
+    if (!cookie)
+        return NULL;
     int cookieLen = cookie->valLen;
     char *pcookieval = pReq->getHeaderBuf().getp(cookie->valOff);
 
@@ -1135,8 +1153,8 @@ static const char *const s_pHeaders[] =
     "if-modified-since",
     "if-match",
     "if-none-match",
-    "if-unmodified-since",
     "if-range",
+    "if-unmodified-since",
     "keep-alive",
     "range",
     "x-forwarded-for",
@@ -1148,7 +1166,7 @@ static const char *const s_pHeaders[] =
 const char *RequestVars::getHeaderString(int iIndex)
 {
     if ((iIndex >= 0)
-        && (iIndex <= (int)(sizeof(s_pHeaders) / sizeof(char *))))
+        && (iIndex < (int)(sizeof(s_pHeaders) / sizeof(char *))))
         return s_pHeaders[iIndex];
     return NULL;
 }
